/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_ADDR_TIMESTAMP_ID_HH_
# define WIFI_FRAME_ADDR_TIMESTAMP_ID_HH_

# include <boost/optional.hpp>

# include <trace-tools/wifi/frame/unique_id/unique_id.hh>
# include <trace-tools/wifi/timestamp.hh>
# include <trace-tools/pcap/frame_descriptor.hh>
# include <trace-tools/wifi/addr_mapping.hh>
# include <trace-tools/wifi/frame/dissector/dissector.hh>
# include <trace-tools/wifi/frame/unique_id/helper_hooks.hh>
# include <trace-tools/wifi/frame/mgt.hh>

namespace wifi
{

  namespace frame
  {

    namespace internals
    {

      struct dst_tmp_tag {};
      struct src_tmp_tag {};
      struct bss_tmp_tag {};

    } // End of namespace wifi::frame::internals.

# define tt_define_addr_timestamp_id(AddrName, AddrIdx, TagName)	\
									\
    typedef								\
      unique_id<internals::adapter<unsigned, TagName>, timestamp>	\
      AddrName ## _timestamp_id;					\
									\
    template <>								\
    boost::optional<AddrName ## _timestamp_id>				\
    make_unique_id<AddrName ## _timestamp_id, prism::header>		\
    (const pcapxx::frame_descriptor&	desc,				\
     addr_mapping&			mapping,			\
     bool				prism_noise_means_error)	\
    {									\
      tt_make_unique_id_prism_common(desc, prism_noise_means_error,	\
				     d, internals::id_helper_hooks, h);	\
									\
      if (const timestamp* t = h.timestamp_ptr())			\
	{								\
	  const addr* a = h.addr_ptr(AddrIdx);				\
									\
	  assert(a);							\
	  return AddrName ## _timestamp_id (desc.pcap_header()->id,	\
					    mapping[*a], *t);		\
	}								\
      return boost::none_t ();						\
    }

    tt_define_addr_timestamp_id(dest,
				mgt::header::da,
				internals::dst_tmp_tag)

    tt_define_addr_timestamp_id(source,
				mgt::header::sa,
				internals::src_tmp_tag)

    tt_define_addr_timestamp_id(bssid,
				mgt::header::bssid,
				internals::bss_tmp_tag)

# undef tt_define_addr_timestamp_id

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_SOURCE_TIMESTAMP_ID_HH_
