/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_SEQCTL_DEST_BSSID_TIMESTAMP_ID_HXX_
# define WIFI_FRAME_SEQCTL_DEST_BSSID_TIMESTAMP_ID_HXX_

# include "seqctl_dest_bssid_timestamp.hh"

# include <trace-tools/wifi/frame/dissector/dissector.hh>
# include <trace-tools/wifi/frame/unique_id/helper_hooks.hh>

namespace wifi
{

  namespace frame
  {

    template <>
    inline
    boost::optional<seqctl_dest_bssid_timestamp_id>
    make_unique_id<seqctl_dest_bssid_timestamp_id,
		   prism::header>(const pcapxx::frame_descriptor& desc,
				  addr_mapping&	mapping,
				  bool prism_noise_means_error)
    {
      tt_make_unique_id_prism_common(desc, prism_noise_means_error,
				     d, internals::id_helper_hooks, h);

      if (const timestamp* tstamp = h.timestamp_ptr())
	{
	  const addr*		da = h.addr_ptr(mgt::header::da);
	  const addr*		bssid = h.addr_ptr(mgt::header::bssid);
	  const uint16_t*	seqctl = h.seq_ctl_ptr();

	  assert(da and bssid and seqctl);

	  return seqctl_dest_bssid_timestamp_id (desc.pcap_header()->id,
						 *seqctl,
						 mapping[*da],
						 mapping[*bssid],
						 *tstamp);
	}
      return boost::none_t ();
    }

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_SEQCTL_DEST_BSSID_TIMESTAMP_ID_HXX_

